@@ -1,28 +1,57 @@
 from abc import ABC
+from typing import List
 
 import pandas as pd
+from ipyaggrid import Grid
 import plotly.express as px
 import plotly.graph_objects as go
 import matplotlib.pyplot as plt
 
+from magnumapi.geometry.Area import Area
+from magnumapi.geometry.Block import Block
 from magnumapi.geometry.Line import Line
 
 
 class Geometry(ABC):
-    def __init__(self, blocks):
+    """ Abstract geometry class providing a skeleton for all geometry types (CosTheta, Rectangular, etc.).
+
+    """
+    def __init__(self, blocks: List[Block]) -> None:
+        """ Constructor of a Geometry class
+
+        :param blocks: a list of instances of Block class implementations (e.g., RectangularBlock, CosThetaBlock, etc.)
+        """
         self.blocks = blocks
 
-    def build_blocks(self):
+    def build_blocks(self) -> None:
+        """ Method building all blocks for a given geometry definition
+
+        """
         for block in self.blocks:
             block.build_block()
 
-    def to_roxie_df(self):
+    def to_roxie_df(self) -> pd.DataFrame:
+        """ Method concatenates row definition of each block into a ROXIE-compatible dataframe
+
+        :return: a concatenated dataframe with ROXIE block definition for a geometry instance
+        """
         return pd.concat([block.to_roxie_df() for block in self.blocks], axis=0).reset_index(drop=True)
 
-    def to_df(self):
+    def to_df(self) -> pd.DataFrame:
+        """ Method concatenating dataframes with area coordinates for each block
+
+        :return: a concatenated dataframe with area coordinates
+        """
         return pd.concat([block.to_df() for block in self.blocks], axis=0).reset_index(drop=True)
 
-    def plot_blocks(self, figsize=(10, 10), is_grid=True, xlim=(0, 80), ylim=(0, 80)):
+    def plot_blocks(self, figsize=(10, 10), is_grid=True, xlim=(0, 80), ylim=(0, 80)) -> None:
+        """ Method plotting all blocks with matplotlib library
+
+        :param figsize: size of a figure on a screen
+        :param is_grid: if True, then the grid is displayed, otherwise False
+        :param xlim: limits in x-direction
+        :param ylim: limits in y-direction
+        """
         fig, ax = plt.subplots(figsize=figsize)
         ax.set_xlim(xlim)
         ax.set_ylim(ylim)
@@ -33,10 +62,16 @@ class Geometry(ABC):
 
         plt.show()
 
-    def plotly_blocks(self, figsize=(65, 65), xlim=(0, 80), ylim=(0, 80)):
-        go_scatter = Geometry.create_plotly_scatters(self.blocks)
+    def plotly_blocks(self, figsize=(65, 65), xlim=(0, 80), ylim=(0, 80)) -> None:
+        """ Method plotting blocks with plotly library
 
-        fig = go.Figure(go_scatter)
+        :param figsize: size of the figure on a display
+        :param xlim: limits in x-direction
+        :param ylim: limits in y-direction
+        """
+        go_scatters = Geometry.create_plotly_scatters(self.blocks)
+
+        fig = go.Figure(go_scatters)
         fig.update_layout(
             autosize=False,
             width=750,
@@ -52,7 +87,12 @@ class Geometry(ABC):
         fig.show()
 
     @staticmethod
-    def create_plotly_scatters(blocks):
+    def create_plotly_scatters(blocks: List[Block]) -> List[go.Scatter]:
+        """ Static method creating a list of plotly scatter instances
+
+        :param blocks: list of blocks to convert to plotly scatter
+        :return: a list of plotly scatter instances
+        """
         go_scatter = []
         index = 1
         for block in blocks:
@@ -71,10 +111,17 @@ class Geometry(ABC):
                 index += 1
         return go_scatter
 
-    def plot_bare_blocks(self, figsize=(15, 15), grid=True, xlim=(0, 80), ylim=(0, 80)):
+    def plot_bare_blocks(self, figsize=(15, 15), is_grid=True, xlim=(0, 80), ylim=(0, 80)) -> None:
+        """ Method plotting bare (uninsulated) areas for all blocks in a geometry
+
+        :param figsize: size of a figure on a screen
+        :param is_grid: if True, then the grid is displayed, otherwise False
+        :param xlim: limits in x-direction
+        :param ylim: limits in y-direction
+        """
         fig, ax = plt.subplots(figsize=figsize)
         ax.set_aspect('equal', 'box')
-        ax.grid(grid)
+        ax.grid(is_grid)
         ax.set_xlim(xlim)
         ax.set_ylim(ylim)
         for block in self.blocks:
@@ -82,27 +129,35 @@ class Geometry(ABC):
 
         plt.show()
 
-    def get_bare_blocks(self):
-        # ToDo - the block object should contain ins_areas and bare_areas
-        bare_blocks = []
-        for block in self.blocks:
-            bare_blocks.append(block.get_bare_areas())
-        return bare_blocks
+    def get_bare_areas_for_blocks(self) -> List[List[Area]]:
+        """ Method returning a list of list of bare (uninsulated) areas constructed from iterating over all blocks in a
+        geometry and all areas in a block.
 
-    def print_blocks(self):
-        for block in self.blocks:
-            block.print_block()
+        :return: a list of list of bare (uninsulated) areas
+        """
+        return [block.get_bare_areas() for block in self.blocks]
 
-    def is_outside_of_first_quadrant(self):
+    def is_outside_of_first_quadrant(self) -> True:
+        """ Method checking whether a geometry is fully positioned in the first quadrant.
+
+        :param area: an Area instance for which the check is performed
+        :return: True if an Area instance is in the first quadrant, False otherwise
+        """
+        # ToDo: Would require an adjustment once layers are introduced.
         for block in self.blocks:
             if block.is_outside_of_first_quadrant():
                 return True
 
         return False
 
-    def are_turns_overlapping(self):
+    def are_turns_overlapping(self) -> bool:
+        """Method checking whether turns from neighbouring blocks are overlapping.
+
+        :return: True, if at least two neighbouring turns are overlapping, otherwise False
+        """
         radius_prev = None
         for index, block in enumerate(self.blocks):
+            # ToDo Fixed once layers are implemented
             if radius_prev == block.block_def.radius:
                 area_curr = block.areas[0]
                 area_prev = self.blocks[index - 1].areas[-1]
@@ -118,9 +173,16 @@ class Geometry(ABC):
 
         return False
 
-    def is_wedge_tip_too_sharp(self, min_value_in_mm):
+    def is_wedge_tip_too_sharp(self, min_value_in_mm: float) -> bool:
+        """ Method checking whether wedge tip is too sharp. Method computes the wedge tip arc length, by finding an
+        intersection of two neighbouring turns with an inner radius
+
+        :param min_value_in_mm: minimum value of  the wedge tip arc
+        :return: True if wedge length is below the minimum value
+        """
         radius_prev = None
         for index, block in enumerate(self.blocks):
+            # ToDo Fixed once layers are implemented
             if radius_prev == block.block_def.radius:
                 area_curr = block.areas[0]
                 area_prev = self.blocks[index - 1].areas[-1]
@@ -137,9 +199,12 @@ class Geometry(ABC):
         return False
 
     @staticmethod
-    def display_definition_table(df):
-        from ipyaggrid import Grid
+    def display_definition_table(df: pd.DataFrame) -> Grid:
+        """ Static method displaying a definition table in an editable format in a notebook
 
+        :param df: a dataframe to be displayed
+        :return: an object to be displayed in a notebook
+        """
         column_defs = [{'headername': c, 'field': c} for c in df.columns]
 
         grid_options = {
