@@ -8,14 +8,11 @@ class InsulationDefinition(Definition):
     """Class for insulation definition.
 
        Attributes:
-           thickness (float): The thickness (mm) on narrow side of cable
+           thickness (float): The thickness (mm) on narrow side of cable.
            width (float): The width (mm) on broad side of cable.
-
     """
-    name: str
     thickness: float
     width: float
-    comment: str
 
     @staticmethod
     def get_magnum_to_roxie_dct() -> dict:
