@@ -4,6 +4,9 @@ from magnumapi.tool_adapters.roxie import RoxieAPI
 
 
 class RoxieInputBuilder:
+    """ Class RoxieInputBuilder builds a ROXIE input
+
+    """
 
     comment = ''
     bhdata_path = ''
@@ -30,6 +33,12 @@ class RoxieInputBuilder:
     brick = pd.DataFrame()
 
     def set_flag(self, flag_name: str, flag_value: bool) -> "RoxieInputBuilder":
+        """ Method setting a flag in a ROXIE input file. An error is thrown if a flag does not exist
+
+        :param flag_name: name of a flag
+        :param flag_value: value of a flag
+        :return: an updated RoxieInputBuilder instance
+        """
         if flag_name in self.flags.keys():
             self.flags[flag_name] = flag_value
         else:
@@ -37,6 +46,11 @@ class RoxieInputBuilder:
         return self
 
     def build(self, template_path: str, output_path: str) -> None:
+        """ Method building a ROXIE input based on a template file
+
+        :param template_path: path to a template file
+        :param output_path: an output path for the input .data file
+        """
         output_str = self.prepare_data_file_str()
 
         # Read template file
@@ -47,6 +61,10 @@ class RoxieInputBuilder:
             input_file.write(template_content.format(output_str))
 
     def prepare_data_file_str(self) -> str:
+        """ Method preparing a data file string content
+
+        :return: a string with lines for an input data file
+        """
         outputs = ['VERSION 10.2.1',
                    '\'%s\'' % self.comment,
                    '\'%s\'' % self.bhdata_path,
@@ -73,12 +91,12 @@ class RoxieInputBuilder:
         return '\n'.join(outputs)
 
     @staticmethod
-    def append_to_outputs(outputs, keyword, df):
-        outputs.append('')
-        outputs.append(RoxieAPI.convert_bottom_header_table_to_str(df, keyword))
+    def convert_flag_dct_to_str(flags: dict) -> str:
+        """ Static method converting a dictionary with flags into a formatted string
 
-    @staticmethod
-    def convert_flag_dct_to_str(flags):
+        :param flags: a dictionary with flags
+        :return: a formatted string representation of the dictionary with flags
+        """
         COLUMN_WIDTH = 11
         flag_per_line_count = 1
         flag_str = '  '
@@ -94,3 +112,14 @@ class RoxieInputBuilder:
 
         flag_str += '\n  /'
         return flag_str
+
+    @staticmethod
+    def append_to_outputs(outputs, keyword, df) -> None:
+        """ Static method appending to the output list (in place) a dataframe with a keyword
+
+        :param outputs: a list of outputs for an input file
+        :param keyword: a table keyword
+        :param df: a dataframe
+        """
+        outputs.append('')
+        outputs.append(RoxieAPI.convert_bottom_header_table_to_str(df, keyword))
