@@ -41,13 +41,13 @@ class TestCosThetaBlock(TestCase):
                                       insul_def=self.cadata.get_insul_definition('16TIL6B'),
                                       strand_def=self.cadata.get_strand_definition('16TIL6B'),
                                       conductor_def=self.cadata.get_conductor_definition('16TIL6B'))
-        p_ref = Point.of_polar(block.block_def.radius, block.phi_r)
+        p_ref = Point.of_polar(block.block_def.radius, block.get_phi())
         area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref,
-                                                           alpha_ref=block.alpha_r)
+                                                           alpha_ref=block.get_alpha())
         area_shift = area_ins.copy()
 
         if block.is_area_intersecting_or_within_radius(area_ins):
-            area_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.alpha_r))
+            area_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.get_phi()))
             shift = Area.find_trapezoid_shift_to_intercept_with_radius(block.block_def.radius, area_shift)
             area_shift = area_shift.translate(area_ins.get_line(0).unit() * shift)
 
@@ -145,7 +145,7 @@ class TestCosThetaBlock(TestCase):
                                       conductor_def=self.cadata.get_conductor_definition('16TIL6B'))
         p_ref = Point.of_polar(block.block_def.radius, block.block_def.phi_r)
         area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref,
-                                                           alpha_ref=block.alpha_r)
+                                                           alpha_ref=block.get_alpha())
         area_shift = area_ins.copy()
 
         fig, ax = plt.subplots(figsize=(15, 15))
@@ -240,13 +240,13 @@ class TestCosThetaBlock(TestCase):
                                       insul_def=self.cadata.get_insul_definition('16TIL6C'),
                                       strand_def=self.cadata.get_strand_definition('16TIL6B'),
                                       conductor_def=self.cadata.get_conductor_definition('16TIL6B'))
-        p_ref = Point.of_polar(block.block_def.radius, block.phi_r)
+        p_ref = Point.of_polar(block.block_def.radius, block.get_phi())
         area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref,
-                                                           alpha_ref=block.alpha_r)
+                                                           alpha_ref=block.get_alpha())
         area_shift = area_ins.copy()
 
         if block.is_area_intersecting_or_within_radius(area_ins):
-            area_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.alpha_r))
+            area_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.get_alpha()))
             shift = Area.find_trapezoid_shift_to_intercept_with_radius(block.block_def.radius, area_shift)
             area_shift = area_shift.translate(area_ins.get_line(0).unit() * shift)
 
@@ -350,11 +350,11 @@ class TestCosThetaBlock(TestCase):
                                       conductor_def=self.cadata.get_conductor_definition('16TIL6B'))
         p_ref = Point.of_polar(block.block_def.radius, block.block_def.phi_r)
         area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref,
-                                                           alpha_ref=block.alpha_r)
+                                                           alpha_ref=block.get_alpha())
         area_shift = area_ins.copy()
 
         if block.is_area_intersecting_or_within_radius(area_ins):
-            area_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.alpha_r))
+            area_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.get_alpha()))
             shift = Area.find_trapezoid_shift_to_intercept_with_radius(block.block_def.radius, area_shift)
             area_shift = area_shift.translate(area_ins.get_line(0).unit() * shift)
 
@@ -456,13 +456,13 @@ class TestCosThetaBlock(TestCase):
                                       insul_def=self.cadata.get_insul_definition('16TIL6C'),
                                       strand_def=self.cadata.get_strand_definition('16TIL6B'),
                                       conductor_def=self.cadata.get_conductor_definition('16TIL6B'))
-        p_ref = Point.of_polar(block.block_def.radius, block.phi_r)
+        p_ref = Point.of_polar(block.block_def.radius, block.get_phi())
         area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref,
-                                                           alpha_ref=block.alpha_r)
+                                                           alpha_ref=block.get_alpha())
         area_shift = area_ins.copy()
 
         if block.is_area_intersecting_or_within_radius(area_ins):
-            area_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.alpha_r))
+            area_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.get_alpha()))
             shift = Area.find_trapezoid_shift_to_intercept_with_radius(block.block_def.radius, area_shift, eps=1e-9)
             area_shift = area_shift.translate(area_ins.get_line(0).unit() * shift)
 
@@ -565,13 +565,13 @@ class TestCosThetaBlock(TestCase):
                                       insul_def=self.cadata.get_insul_definition('16TIL6D'),
                                       strand_def=self.cadata.get_strand_definition('16TIL6D'),
                                       conductor_def=self.cadata.get_conductor_definition('16TIL6D'))
-        p_ref = Point.of_polar(block.block_def.radius, block.phi_r)
+        p_ref = Point.of_polar(block.block_def.radius, block.get_phi())
         area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref,
-                                                           alpha_ref=block.alpha_r)
+                                                           alpha_ref=block.get_alpha())
         area_shift = area_ins.copy()
 
         if block.is_area_intersecting_or_within_radius(area_ins):
-            area_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.alpha_r))
+            area_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.get_alpha()))
             shift = Area.find_trapezoid_shift_to_intercept_with_radius(block.block_def.radius, area_shift)
             area_shift = area_shift.translate(area_ins.get_line(0).unit() * shift)
 
@@ -667,9 +667,9 @@ class TestCosThetaBlock(TestCase):
                                       insul_def=self.cadata.get_insul_definition('16TIL6E'),
                                       strand_def=self.cadata.get_strand_definition('16TIL6E'),
                                       conductor_def=self.cadata.get_conductor_definition('16TIL6E'))
-        p_ref = Point.of_polar(block.block_def.radius, block.phi_r)
+        p_ref = Point.of_polar(block.block_def.radius, block.get_phi())
         area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref,
-                                                           alpha_ref=block.alpha_r)
+                                                           alpha_ref=block.get_alpha())
         area_shift = area_ins.copy()
 
         fig, ax = plt.subplots(figsize=(15, 15))
@@ -765,11 +765,11 @@ class TestCosThetaBlock(TestCase):
                                       strand_def=self.cadata.get_strand_definition('16TIL6E'),
                                       conductor_def=self.cadata.get_conductor_definition('16TIL6E'))
 
-        p_ref = Point.of_polar(block.block_def.radius, block.phi_r)
-        area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref, alpha_ref=block.alpha_r)
+        p_ref = Point.of_polar(block.block_def.radius, block.get_phi())
+        area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref, alpha_ref=block.get_alpha())
         area_ins_shift = area_ins.copy()
         if block.is_area_intersecting_or_within_radius(area_ins):
-            area_ins_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.alpha_r))
+            area_ins_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.get_alpha()))
             shift = Area.find_trapezoid_shift_to_intercept_with_radius(block.block_def.radius, area_ins_shift, eps=1e-9)
             area_ins_shift = area_ins_shift.translate(area_ins.get_line(0).unit() * shift)
 
@@ -867,11 +867,11 @@ class TestCosThetaBlock(TestCase):
                                       strand_def=self.cadata.get_strand_definition('16TIL6E'),
                                       conductor_def=self.cadata.get_conductor_definition('16TIL6E'))
 
-        p_ref = Point.of_polar(block.block_def.radius, block.phi_r)
-        area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref, alpha_ref=block.alpha_r)
+        p_ref = Point.of_polar(block.block_def.radius, block.get_phi())
+        area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref, alpha_ref=block.get_alpha())
         area_ins_shift = area_ins.copy()
         if block.is_area_intersecting_or_within_radius(area_ins):
-            area_ins_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.alpha_r))
+            area_ins_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.get_alpha()))
             shift = Area.find_trapezoid_shift_to_intercept_with_radius(block.block_def.radius, area_ins_shift, eps=1e-6)
             area_ins_shift = area_ins_shift.translate(area_ins.get_line(0).unit() * shift)
 
@@ -970,11 +970,11 @@ class TestCosThetaBlock(TestCase):
                                       strand_def=self.cadata.get_strand_definition('16TIL6E'),
                                       conductor_def=self.cadata.get_conductor_definition('16TIL6E'))
 
-        p_ref = Point.of_polar(block.block_def.radius, block.phi_r)
-        area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref, alpha_ref=block.alpha_r)
+        p_ref = Point.of_polar(block.block_def.radius, block.get_phi())
+        area_ins = block.get_insulated_isosceles_trapezium(p_ref=p_ref, alpha_ref=block.get_alpha())
         area_ins_shift = area_ins.copy()
         if block.is_area_intersecting_or_within_radius(area_ins):
-            area_ins_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.alpha_r))
+            area_ins_shift = area_ins.translate(Point.of_polar(2 * block.cable_def.thickness_i, block.get_alpha()))
             shift = Area.find_trapezoid_shift_to_intercept_with_radius(block.block_def.radius, area_ins_shift, eps=1e-6)
             area_ins_shift = area_ins_shift.translate(area_ins.get_line(0).unit() * shift)
 
@@ -1096,27 +1096,3 @@ class TestCosThetaBlock(TestCase):
             cos_theta_block.to_roxie_df()
 
         self.assertTrue('This method is not implemented for this class' in str(context.exception))
-
-    def test_print_block(self):
-        # arrange
-        block_rel_def = RelativeCosThetaBlockDefinition(radius=25, phi_r=5.0, alpha_r=10.0, nco=1, condname='16TIL6E',
-                                                        no=1, current=13500, type=1, n1=2, n2=20, imag=0, turn=0)
-        block = RelativeCosThetaBlock(block_def=block_rel_def,
-                                      cable_def=self.cadata.get_cable_definition('16TIL6E'),
-                                      insul_def=self.cadata.get_insul_definition('16TIL6E'),
-                                      strand_def=self.cadata.get_strand_definition('16TIL6E'),
-                                      conductor_def = self.cadata.get_conductor_definition('16TIL6E'))
-
-        # act
-        block.build_block()
-
-        # assert
-        with patch('sys.stdout', new=StringIO()) as fake_out:
-            block.print_block()
-
-        print_str = '(25.017258, 2.198711) -> (46.821732, 6.043428),' \
-                    '(46.821732, 6.043428) -> (46.354814, 8.562654),' \
-                    '(46.354814, 8.562654) -> (24.620782, 4.337872),' \
-                    '(24.620782, 4.337872) -> (25.017258, 2.198711)'
-
-        self.assertEqual(print_str, fake_out.getvalue().strip())
