@@ -701,7 +701,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         # x(4) = 46.2349
         # y(4) = 8.3865
 
-        bare_blocks = geometry.get_bare_blocks()
+        bare_blocks = geometry.get_bare_areas_for_blocks()
 
         self.assertAlmostEqual(24.7954, round(bare_blocks[0][0].get_line(3).p1.x, 4), places=4)
         self.assertAlmostEqual(4.2190, round(bare_blocks[0][0].get_line(3).p1.y, 4), places=4)
@@ -759,7 +759,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         # x(4) = 45.1392
         # y(4) = 13.3866
 
-        bare_blocks = geometry.get_bare_blocks()
+        bare_blocks = geometry.get_bare_areas_for_blocks()
 
         self.assertAlmostEqual(23.8582, round(bare_blocks[0][-1].get_line(3).p1.x, 4), places=4)
         self.assertAlmostEqual(8.4732, round(bare_blocks[0][-1].get_line(3).p1.y, 4), places=4)
@@ -837,7 +837,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         # x(4) = 46.2349
         # y(4) = 8.3865
 
-        bare_blocks = geometry.get_bare_blocks()
+        bare_blocks = geometry.get_bare_areas_for_blocks()
 
         self.assertAlmostEqual(24.7954, round(bare_blocks[0][0].get_line(3).p1.x, 4), places=4)
         self.assertAlmostEqual(4.2190, round(bare_blocks[0][0].get_line(3).p1.y, 4), places=4)
@@ -895,7 +895,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         # x(4) = 45.1392
         # y(4) = 13.3866
 
-        bare_blocks = geometry.get_bare_blocks()
+        bare_blocks = geometry.get_bare_areas_for_blocks()
 
         self.assertAlmostEqual(23.8582, round(bare_blocks[0][-1].get_line(3).p1.x, 4), places=4)
         self.assertAlmostEqual(8.4732, round(bare_blocks[0][-1].get_line(3).p1.y, 4), places=4)
@@ -1056,28 +1056,6 @@ class CosThetaGeometryTest(unittest.TestCase):
         geometry.build_blocks()
         geometry.display_definition_table(geometry.to_df())
 
-    def test_simplified_magnet_input_print_blocks(self):
-        # arrange
-        blocks_def = [{'no': 1, 'radius': 25.0, 'alpha_r': 0, 'phi_r': 0.57294, 'nco': 1, 'type': 1, 'current': 13500,
-                       'condname': '16TIL9', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}]
-
-        cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
-        cadata = CableDatabase.read_cadata(cadata_path)
-
-        # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
-        geometry.build_blocks()
-
-        with patch('sys.stdout', new=StringIO()) as fake_out:
-            geometry.print_blocks()
-
-        print_str = '(24.998750, 0.249988) -> (38.348877, 0.249988),' \
-                    '(38.348877, 0.249988) -> (38.339461, 2.407823),' \
-                    '(38.339461, 2.407823) -> (24.989842, 2.291316),' \
-                    '(24.989842, 2.291316) -> (24.998750, 0.249988)'
-
-        self.assertEqual(print_str, fake_out.getvalue().strip())
-
     def test_with_roxie_absolute_angle_definition_json(self):
         # arrange
         json_path = create_resources_file_path('resources/geometry/roxie/16T/16T_abs.json')
